// Copyright 2011 Thomas Kamps anubis1@linux-ecke.de
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

#include "cookiejar.h"
#include <QDir>
#include <QFile>

#include <QDateTime>
#include <QDebug>

CookieJar::CookieJar(QObject *parent) : QNetworkCookieJar(parent) {
	//Read allowed cookiedomains
	QFile allowedDomains(QDir::homePath()+"/.lightweight2/whitelistCookies");
	if (allowedDomains.exists()) {
		allowedDomains.open(QFile::ReadOnly);
		while (allowedDomains.bytesAvailable()) {
			QByteArray arr = allowedDomains.readLine();
			QString domain = QString::fromUtf8(arr);
			if (domain.endsWith("\n"))
				domain.chop(1);
			if (domain.size() < 1) continue;
			qDebug() << "Whitelisted cookiedomain: " << domain;
			allowedCookieDomains.append(domain);
		}
		allowedDomains.close();
	}

	//Read saved cookies
	QFile cookies(QDir::homePath()+"/.lightweight2/cookies");
	if (cookies.exists()) {
		QList<QNetworkCookie> cookieList;
		cookies.open(QFile::ReadOnly);
		while (cookies.bytesAvailable()) {
			QByteArray arr = cookies.readLine();
			QString cookie = QString::fromUtf8(arr);
			if (cookie.endsWith("\n")) cookie.chop(1);
			cookieList.append(QNetworkCookie::parseCookies(cookie.toUtf8()));
		}
		cookies.close();
		setAllCookies(cookieList);
	}
}

void CookieJar::saveCookies() {
	QFile cookieFile(QDir::homePath()+"/.lightweight2/cookies");
	cookieFile.open(QFile::WriteOnly);

	//Save cookies, if there are in the allowesCookieDomains-list
	QList<QNetworkCookie> cookies = allCookies();

	for (int i = 0; i < cookies.size(); i++) {
		bool allowed = false;

		//Check if cookies may not be saved
		if (cookies.value(i).isSessionCookie()) continue;
		if (cookies.value(i).expirationDate() < QDateTime::currentDateTime()) continue;

		//Only save non session cookies and is not expirated
		foreach (QString d, allowedCookieDomains) {
			if (cookies.value(i).domain().endsWith(d)) {
				allowed = true;
				break;
			}
		}

		if (allowed) {
			cookieFile.write(cookies.value(i).toRawForm()+"\n");
			qDebug() << "Saved cookie from: " << cookies.value(i).domain();
		} else {
			qDebug() << "Discarded cookie from: " << cookies.value(i).domain();
		}
	}
	cookieFile.flush();
	cookieFile.close();
}
